@@ -32,6 +32,7 @@ import textwrap
 import pkg_resources
 import six
 
+from oslo_config._i18n import _LW
 from oslo_config import cfg
 import stevedore.named  # noqa
 
@@ -47,6 +48,8 @@ _generator_opts = [
                     required=True,
                     help='Option namespace under "oslo.config.opts" in which '
                          'to query for options.'),
+    cfg.BoolOpt('minimal', default=False,
+                help='Generate a minimal required configuration.'),
 ]
 
 
@@ -76,9 +79,8 @@ def _format_defaults(opt):
             default_str = str(opt.sample_default)
         elif opt.default is None:
             default_str = '<None>'
-        elif (isinstance(opt, cfg.StrOpt) or
-              isinstance(opt, cfg.IPOpt) or
-              isinstance(opt, cfg.HostnameOpt)):
+        elif (isinstance(opt, (cfg.StrOpt, cfg.IPOpt,
+                               cfg.HostnameOpt, cfg.URIOpt))):
             default_str = opt.default
         elif isinstance(opt, cfg.BoolOpt):
             default_str = str(opt.default).lower()
@@ -93,7 +95,7 @@ def _format_defaults(opt):
                                   key=operator.itemgetter(0))
             default_str = ','.join(['%s:%s' % i for i in sorted_items])
         else:
-            LOG.warning('Unknown option type: %s', repr(opt))
+            LOG.warning(_LW('Unknown option type: %s'), repr(opt))
             default_str = str(opt.default)
         defaults = [default_str]
 
@@ -105,6 +107,28 @@ def _format_defaults(opt):
     return results
 
 
+_TYPE_NAMES = {
+    str: 'string value',
+    int: 'integer value',
+    float: 'floating point value',
+}
+
+
+def _format_type_name(opt_type):
+    """Format the type name to use in describing an option"""
+    try:
+        return opt_type.type_name
+    except AttributeError:  # nosec
+        pass
+
+    try:
+        return _TYPE_NAMES[opt_type]
+    except KeyError:  # nosec
+        pass
+
+    return 'unknown value'
+
+
 class _OptFormatter(object):
 
     """Format configuration option descriptions to a file."""
@@ -160,20 +184,24 @@ class _OptFormatter(object):
             lines = ['[%s]\n' % groupname]
         self.writelines(lines)
 
-    def format(self, opt):
+    def format(self, opt, group_name, minimal=False):
         """Format a description of an option to the output file.
 
         :param opt: a cfg.Opt instance
         """
         if not opt.help:
-            LOG.warning('"%s" is missing a help string', opt.dest)
+            LOG.warning(_LW('"%s" is missing a help string'), opt.dest)
 
-        option_type = getattr(opt, 'type', None)
-        opt_type = getattr(option_type, 'type_name', 'unknown value')
+        opt_type = _format_type_name(opt.type)
+        opt_prefix = ''
+        if (opt.deprecated_for_removal and
+                not opt.help.startswith('DEPRECATED')):
+            opt_prefix = 'DEPRECATED: '
 
         if opt.help:
-            help_text = u'%s (%s)' % (opt.help,
-                                      opt_type)
+            help_text = u'%s%s (%s)' % (opt_prefix,
+                                        opt.help,
+                                        opt_type)
         else:
             help_text = u'(%s)' % opt_type
         lines = self._format_help(help_text)
@@ -204,7 +232,7 @@ class _OptFormatter(object):
             # https://bugs.launchpad.net/keystoneauth/+bug/1548433 for
             # more details.
             import warnings
-            if not isinstance(cfg.Opt, opt):
+            if not isinstance(opt, cfg.Opt):
                 warnings.warn(
                     'Incompatible option class for %s (%r): %s' %
                     (opt.dest, opt.__class__, err),
@@ -215,16 +243,29 @@ class _OptFormatter(object):
 
         for d in opt.deprecated_opts:
             lines.append('# Deprecated group/name - [%s]/%s\n' %
-                         (d.group or 'DEFAULT', d.name or opt.dest))
+                         (d.group or group_name, d.name or opt.dest))
 
         if opt.deprecated_for_removal:
+            if opt.deprecated_since:
+                lines.append(
+                    '# This option is deprecated for removal since %s.\n' % (
+                        opt.deprecated_since))
+            else:
+                lines.append(
+                    '# This option is deprecated for removal.\n')
             lines.append(
-                '# This option is deprecated for removal.\n'
                 '# Its value may be silently ignored in the future.\n')
             if opt.deprecated_reason:
                 lines.extend(
                     self._format_help('Reason: ' + opt.deprecated_reason))
 
+        if opt.advanced:
+            lines.append(
+                '# Advanced Option: intended for advanced users and not used\n'
+                '# by the majority of users, and might have a significant\n'
+                '# effect on stability and/or performance.\n'
+            )
+
         if hasattr(opt.type, 'format_defaults'):
             defaults = opt.type.format_defaults(opt.default,
                                                 opt.sample_default)
@@ -239,7 +280,10 @@ class _OptFormatter(object):
         for default_str in defaults:
             if default_str:
                 default_str = ' ' + default_str
-            lines.append('#%s =%s\n' % (opt.dest, default_str))
+            if minimal:
+                lines.append('%s =%s\n' % (opt.dest, default_str))
+            else:
+                lines.append('#%s =%s\n' % (opt.dest, default_str))
 
         self.writelines(lines)
 
@@ -347,38 +391,54 @@ def on_load_failure_callback(*args, **kwargs):
     raise
 
 
-def _output_opts(f, group, namespaces):
-    f.format_group(group)
-    for (namespace, opts) in sorted(namespaces,
+def _output_opts(f, group, group_data, minimal=False):
+    f.format_group(group_data['object'] or group)
+    for (namespace, opts) in sorted(group_data['namespaces'],
                                     key=operator.itemgetter(0)):
         f.write('\n#\n# From %s\n#\n' % namespace)
-        for opt in opts:
-            f.write('\n')
+        for opt in sorted(opts, key=operator.attrgetter('advanced')):
             try:
-                f.format(opt)
+                if minimal and not opt.required:
+                    pass
+                else:
+                    f.write('\n')
+                    f.format(opt, group, minimal)
             except Exception as err:
                 f.write('# Warning: Failed to format sample for %s\n' %
                         (opt.dest,))
                 f.write('# %s\n' % (err,))
 
 
-def _get_group_name(item):
-    group = item[0]
-    # The keys of the groups dictionary could be an OptGroup. Otherwise the
-    # help text of an OptGroup wouldn't be part of the generated sample
-    # file. It could also be just a plain group name without any further
-    # attributes. That's the reason why we have to differentiate here.
-    return group.name if isinstance(group, cfg.OptGroup) else group
-
-
 def _get_groups(conf_ns):
-    groups = {'DEFAULT': []}
+    """Invert a list of groups by namespace into a dict by group name.
+
+    :param conf_ns: a list of (namespace, [(<group>, [opt_1, opt_2])]) tuples,
+                    such as returned by _list_opts.
+    :returns: {<group_name>, {'object': <group_object>,
+                              'namespaces': [(<namespace>, <opts>)]}}
+
+    <group> may be a string or a group object.
+    <group_name> is always a string.
+    <group_object> will only be set if <group> was a group object in at least
+    one namespace.
+
+    Keying by group_name avoids adding duplicate group names in case a group is
+    added as both an OptGroup and as a str, but still makes the additional
+    OptGroup data available to the output code when possible.
+    """
+    groups = {'DEFAULT': {'object': None, 'namespaces': []}}
     for namespace, listing in conf_ns:
         for group, opts in listing:
             if not opts:
                 continue
-            namespaces = groups.setdefault(group or 'DEFAULT', [])
-            namespaces.append((namespace, opts))
+            group = group if group else 'DEFAULT'
+            is_optgroup = hasattr(group, 'name')
+            group_name = group.name if is_optgroup else group
+            if group_name not in groups:
+                groups[group_name] = {'object': None, 'namespaces': []}
+            if is_optgroup:
+                groups[group_name]['object'] = group
+            groups[group_name]['namespaces'].append((namespace, opts))
     return groups
 
 
@@ -401,12 +461,12 @@ def generate(conf):
     groups = _get_groups(_list_opts(conf.namespace))
 
     # Output the "DEFAULT" section as the very first section
-    _output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'))
+    _output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'), conf.minimal)
 
     # output all other config sections with groups in alphabetical order
-    for group, namespaces in sorted(groups.items(), key=_get_group_name):
+    for group, group_data in sorted(groups.items()):
         formatter.write('\n\n')
-        _output_opts(formatter, group, namespaces)
+        _output_opts(formatter, group, group_data, conf.minimal)
 
 
 def main(args=None):
